@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2017-2022, Intel Corporation
+* Copyright (c) 2017-2024, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -30,10 +30,14 @@
 #include "codec_def_common_avc.h"
 #include "codec_def_common_encode.h"
 #include "codec_def_common.h"
+#include "codec_def_encode.h"
 
 #define CODEC_AVC_NUM_MAX_DIRTY_RECT        4
 #define CODEC_AVC_NUM_QP                    52
 
+#define CODEC_AVC_MAX_QP 51
+#define CODEC_AVC_MIN_QP_LP 10
+
 #define CODEC_AVC_NUM_WP_FRAME              8
 #define CODEC_AVC_MAX_FORWARD_WP_FRAME      6
 #define CODEC_AVC_MAX_BACKWARD_WP_FRAME     2
@@ -57,6 +61,14 @@
 #define CODECHAL_ENCODE_AVC_ROI_FRAME_HEIGHT_SCALE_FACTOR   16
 #define CODECHAL_ENCODE_AVC_ROI_FIELD_HEIGHT_SCALE_FACTOR   32
 #define CODECHAL_ENCODE_AVC_MAX_ROI_NUMBER                  4
+#define CODECHAL_ENCODE_AVC_MAX_SLICE_QP                    (CODEC_AVC_NUM_QP - 1) // 0 - 51 inclusive
+#define CODECHAL_ENCODE_AVC_MAX_ICQ_QUALITYFACTOR           51
+#define CODECHAL_ENCODE_AVC_MIN_ICQ_QUALITYFACTOR           1
+#define CODECHAL_ENCODE_AVC_MAX_SLICES_SUPPORTED            256
+#define CODECHAL_ENCODE_AVC_MIN_STREAM_BUFFER_ALIGNMENT     4096
+#define CODECHAL_DEBUG_ENCODE_AVC_NAL_START_CODE_SEI        0x00000106
+#define CODECHAL_DEBUG_ENCODE_AVC_NAL_START_CODE_PPS        0x00000128
+#define CODECHAL_ENCODE_AVC_MI_COPY_MEM_MEM_CMD_ALIGNMENT   4
 
 typedef struct _CODECHAL_ENCODE_AVC_ROUNDING_PARAMS
 {
@@ -232,19 +244,6 @@ typedef struct _CODEC_ROI_MAP
     CODEC_ROI           Rect[16];   // disconnected areas which have same PriorityLevelOrDQp
 } CODEC_ROI_MAP, *PCODEC_ROI_MAP;
 
-typedef struct _CODEC_ENCODE_MB_CONTROL
-{
-    union
-    {
-        struct
-        {
-            uint32_t        bForceIntra : 1;
-            uint32_t        Reserved : 31;
-        };
-        uint32_t            value;
-    } MBParams;
-} CODEC_ENCODE_MB_CONTROL, *PCODEC_ENCODE_MB_CONTROL;
-
 typedef struct _CODEC_PIC_REORDER
 {
     uint32_t           PicNum;
@@ -1051,6 +1050,34 @@ typedef struct _CODEC_AVC_ENCODE_PIC_PARAMS
     */
     uint8_t         QpModulationStrength;
 
+    /*! \brief StatusReportEnable
+    *
+    *  Request features to be enabled at status report.
+    *  FrameStats: FRAME_STATS_INFO enabled in ENCODE_QUERY_STATUS_PARAMS.
+    *  BlockStats: BLOCK_STATS_INFO enabled in ENCODE_QUERY_STATUS_PARAMS.
+    */
+    union
+    {
+        struct
+        {
+            uint16_t FrameStats : 1;
+            uint16_t BlockStats : 1;
+            uint16_t reserved : 14;
+        } fields;
+        uint16_t value;
+    } StatusReportEnable;
+
+    /*! \brief quality information report enable flags.
+    */
+    union
+    {
+        struct
+        {
+            uint8_t enable_frame : 1;
+            uint8_t reserved : 7;
+        } fields;
+        uint8_t value;
+    } QualityInfoSupportFlags;
 } CODEC_AVC_ENCODE_PIC_PARAMS, *PCODEC_AVC_ENCODE_PIC_PARAMS;
 
 /*! \brief Slice-level parameters of a compressed picture for AVC encoding.
@@ -1200,4 +1227,78 @@ struct CodecEncodeAvcFeiPicParams
     uint32_t                    dwNumPasses;     //number of QPs
     uint8_t                    *pDeltaQp;        //list of detla QPs
 };
+
+typedef struct _CODECHAL_ENCODE_AVC_PACK_PIC_HEADER_PARAMS
+{
+    PBSBuffer                               pBsBuffer;
+    PCODEC_AVC_ENCODE_PIC_PARAMS            pPicParams;     // pAvcPicParams[ucPPSIdx]
+    PCODEC_AVC_ENCODE_SEQUENCE_PARAMS       pSeqParams;     // pAvcSeqParams[ucSPSIdx]
+    PCODECHAL_ENCODE_AVC_VUI_PARAMS         pAvcVuiParams;
+    PCODEC_AVC_IQ_MATRIX_PARAMS             pAvcIQMatrixParams;
+    PCODECHAL_NAL_UNIT_PARAMS               *ppNALUnitParams;
+    CodechalEncodeSeiData*                  pSeiData;
+    uint32_t                                dwFrameHeight;
+    uint32_t                                dwOriFrameHeight;
+    uint16_t                                wPictureCodingType;
+    bool                                    bNewSeq;
+    bool                                   *pbNewPPSHeader;
+    bool                                   *pbNewSeqHeader;
+} CODECHAL_ENCODE_AVC_PACK_PIC_HEADER_PARAMS, *PCODECHAL_ENCODE_AVC_PACK_PIC_HEADER_PARAMS;
+
+typedef struct _CODECHAL_ENCODE_AVC_VALIDATE_NUM_REFS_PARAMS
+{
+    PCODEC_AVC_ENCODE_SEQUENCE_PARAMS       pSeqParams;     // pAvcSeqParams[ucSPSIdx]
+    PCODEC_AVC_ENCODE_PIC_PARAMS            pPicParams;
+    PCODEC_AVC_ENCODE_SLICE_PARAMS          pAvcSliceParams;
+    uint16_t                                wPictureCodingType;
+    uint16_t                                wPicHeightInMB;
+    uint16_t                                wFrameFieldHeightInMB;
+    bool                                    bFirstFieldIPic;
+    bool                                    bVDEncEnabled;
+    bool                                    bPAKonly;
+} CODECHAL_ENCODE_AVC_VALIDATE_NUM_REFS_PARAMS, *PCODECHAL_ENCODE_AVC_VALIDATE_NUM_REFS_PARAMS;
+
+typedef struct _CODECHAL_ENCODE_AVC_TQ_INPUT_PARAMS
+{
+    uint16_t  wPictureCodingType;
+    uint8_t   ucTargetUsage;
+    uint8_t   ucQP;
+    bool      bBrcEnabled;
+    bool      bVdEncEnabled;
+} CODECHAL_ENCODE_AVC_TQ_INPUT_PARAMS, *PCODECHAL_ENCODE_AVC_TQ_INPUT_PARAMS;
+
+typedef struct _CODECHAL_ENCODE_AVC_TQ_PARAMS
+{
+    uint32_t    dwTqEnabled;
+    uint32_t    dwTqRounding;
+} CODECHAL_ENCODE_AVC_TQ_PARAMS, *PCODECHAL_ENCODE_AVC_TQ_PARAMS;
+
+//!
+//! \enum   TrellisSetting
+//! \brief  Indicate the different Trellis Settings
+//!
+enum TrellisSetting
+{
+    trellisInternal = 0,
+    trellisDisabled = 1,
+    trellisEnabledI = 2,
+    trellisEnabledP = 4,
+    trellisEnabledB = 8
+};
+
+typedef struct _CODECHAL_ENCODE_AVC_PACK_SLC_HEADER_PARAMS
+{
+    PBSBuffer                               pBsBuffer;
+    PCODEC_AVC_ENCODE_PIC_PARAMS            pPicParams;     // pAvcPicParams[ucPPSIdx]
+    PCODEC_AVC_ENCODE_SEQUENCE_PARAMS       pSeqParams;     // pAvcSeqParams[ucSPSIdx]
+    PCODEC_AVC_ENCODE_SLICE_PARAMS          pAvcSliceParams;
+    PCODEC_REF_LIST                         *ppRefList;
+    CODEC_PICTURE                           CurrPic;
+    CODEC_PICTURE                           CurrReconPic;
+    CODEC_AVC_ENCODE_USER_FLAGS             UserFlags;
+    CODECHAL_ENCODE_AVC_NAL_UNIT_TYPE       NalUnitType;
+    uint16_t                                wPictureCodingType;
+    bool                                    bVdencEnabled;
+} CODECHAL_ENCODE_AVC_PACK_SLC_HEADER_PARAMS, *PCODECHAL_ENCODE_AVC_PACK_SLC_HEADER_PARAMS;
+
 #endif  // __CODEC_DEF_ENCODE_AVC_H__
