@@ -176,7 +176,11 @@ typedef struct _MHW_VEBOX_MODE
     uint32_t    Hdr1DLutEnable                      : 1;
     uint32_t    Fp16ModeEnable                      : 1;
     uint32_t    Hdr1K1DLut                          : 1;
-    uint32_t                                        : 7; // Reserved
+    uint32_t    GamutExpansionPosition              : 1;
+    uint32_t    EotfPrecision                       : 1;
+    uint32_t    BypassCcm                           : 1;
+    uint32_t    BypassOetf                          : 1;
+    uint32_t                                        : 3; // Reserved
 } MHW_VEBOX_MODE, *PMHW_VEBOX_MODE;
 
 typedef enum _MHW_VEBOX_ADDRESS_SHIFT
@@ -208,6 +212,14 @@ typedef struct _MHW_VEBOX_3D_LUT
     uint32_t                                                : 25; // Reserved
 } MHW_VEBOX_3D_LUT, *PMHW_VEBOX_3D_LUT;
 
+typedef struct _MHW_VEBOX_FP16_INPUT
+{
+    uint32_t VeboxFp16InputEnable                           : 1;
+    uint32_t RgbSwapForFp16Input                            : 1;
+    uint32_t HdrGainFactor                                  : 8;
+    uint32_t                                                : 22;  // Reserved
+} MHW_VEBOX_FP16_INPUT, *PMHW_VEBOX_FP16_INPUT;
+
 //!
 //! \brief  Structure to handle VEBOX_STATE_CMD Command
 //!
@@ -216,6 +228,7 @@ typedef struct _MHW_VEBOX_STATE_CMD_PARAMS
     MHW_VEBOX_MODE                      VeboxMode;
     MHW_VEBOX_CHROMA_SAMPLING           ChromaSampling;
     MHW_VEBOX_3D_LUT                    LUT3D;
+    MHW_VEBOX_FP16_INPUT                FP16Input;
     bool                                bUseVeboxHeapKernelResource;
     PMOS_RESOURCE                       pLaceLookUpTables;
     PMOS_RESOURCE                       pVeboxParamSurf;
@@ -225,6 +238,7 @@ typedef struct _MHW_VEBOX_STATE_CMD_PARAMS
     MHW_MEMORY_OBJECT_CONTROL_PARAMS    LaceLookUpTablesSurfCtrl;
     MHW_MEMORY_OBJECT_CONTROL_PARAMS    Vebox3DLookUpTablesSurfCtrl;
     bool                                bNoUseVeboxHeap;
+    bool                                isTlbPrefetchDisable;
 } MHW_VEBOX_STATE_CMD_PARAMS, *PMHW_VEBOX_STATE_CMD_PARAMS;
 
 //!
@@ -625,6 +639,17 @@ typedef struct _MHW_1DLUT_PARAMS
     uint32_t CCMSize;
 } MHW_1DLUT_PARAMS, *PMHW_1DLUT_PARAMS;
 
+//!
+//! Structure MHW_3DLUT_PARAMS
+//! \details No pre-si version for MHW_VEBOX_IECP_PARAMS, just leave it now and handle it later
+//!
+typedef struct _MHW_FP16_PARAMS
+{
+    uint32_t                isActive       = false;              //!< Active or not
+    uint32_t                OETFLutY[256]  = {};
+    uint32_t                OETFLutX[256]  = {};
+} MHW_FP16_PARAMS, *PMHW_FP16_PARAMS;
+
 //!
 //! \brief  VEBOX IECP parameters
 //!
@@ -650,6 +675,7 @@ typedef struct _MHW_VEBOX_IECP_PARAMS
 
     MHW_3DLUT_PARAMS                s3DLutParams;
     MHW_1DLUT_PARAMS                s1DLutParams;
+    MHW_FP16_PARAMS                 fp16Params;
 
     // Front End CSC params
     bool                            bFeCSCEnable;                               // Enable Front End CSC transform
@@ -659,6 +685,8 @@ typedef struct _MHW_VEBOX_IECP_PARAMS
 
     // FDFB
     bool                            iecpstateforFDFB;
+    // CSC via CCM for float 16bit output
+    bool                            bCcmCscEnable;
 } MHW_VEBOX_IECP_PARAMS, *PMHW_VEBOX_IECP_PARAMS;
 
 //!
