@@ -365,6 +365,7 @@ static bool InitTglMediaWa(struct GfxDeviceInfo *devInfo,
     MEDIA_WR_WA(waTable, Wa_22010493002, 1);
 
     MEDIA_WR_WA(waTable, Wa_Vp9UnalignedHeight, 1);
+    MEDIA_WR_WA(waTable, Wa_AvcUnalignedHeight, 1);
     MEDIA_WR_WA(waTable, WaDisableSetObjectCapture, 0);
     return true;
 }
@@ -522,13 +523,16 @@ static bool InitAdlsMediaSku(struct GfxDeviceInfo *devInfo,
 
     MEDIA_WR_SKU(skuTable, FtrAV1VLDLSTDecoding, 1);
 
+    MEDIA_WR_SKU(skuTable, FtrHeight8AlignVE3DLUTDualPipe, 1);
+
     //Disable VP8 for ADLS
     MEDIA_WR_SKU(skuTable, FtrIntelVP8VLDDecoding, 0);
 
     // Disable HEVC 422 Virtual Tile Scalability
     MEDIA_WR_SKU(skuTable, FtrDecodeHEVC422VTScalaDisable, 1);
-    MEDIA_WR_SKU(skuTable, FtrE2ECompression, 1);
 
+    MEDIA_WR_SKU(skuTable, FtrVirtualTileScalabilityDisable, 1);
+    MEDIA_WR_SKU(skuTable, FtrE2ECompression, 1);
 
     //RPL-S
     if (drvInfo->devId == 0xA780 || 
@@ -597,6 +601,7 @@ static bool InitAdlpMediaSku(struct GfxDeviceInfo *devInfo,
     MEDIA_WR_SKU(skuTable, FtrAV1VLDLSTDecoding, 1);
     MEDIA_WR_SKU(skuTable, FtrGucSubmission, 1);
     MEDIA_WR_SKU(skuTable, FtrDecodeHEVC422VTScalaDisable, 1);
+    MEDIA_WR_SKU(skuTable, FtrVirtualTileScalabilityDisable, 1);
 
     return true;
 }
@@ -613,6 +618,7 @@ static bool InitAdlpMediaWa(struct GfxDeviceInfo *devInfo,
     //ADL-P not need this
     MEDIA_WR_WA(waTable, Wa_1409820462, 0);
     MEDIA_WR_WA(waTable, WaDisableClearCCS, 1);
+
     return true;
 }
 
@@ -643,7 +649,8 @@ static bool InitAdlnMediaSku(struct GfxDeviceInfo *devInfo,
     {
         MEDIA_WR_SKU(skuTable, FtrGT0_5, 1);
     }
-
+    
+    MEDIA_WR_SKU(skuTable, FtrHeight8AlignVE3DLUTDualPipe, 1);
     MEDIA_WR_SKU(skuTable, FtrAV1VLDLSTDecoding, 1);
     MEDIA_WR_SKU(skuTable, FtrGucSubmission, 1);
     MEDIA_WR_SKU(skuTable, FtrDecodeHEVC422VTScalaDisable, 1);
