@@ -1,6 +1,6 @@
 /*===================== begin_copyright_notice ==================================
 
-# Copyright (c) 2021-2022, Intel Corporation
+# Copyright (c) 2021-2024, Intel Corporation
 
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -277,35 +277,38 @@ MOS_STATUS MmdDeviceXe_Hpm::Initialize(
     PMOS_INTERFACE osInterface,
     MhwInterfaces *mhwInterfaces)
 {
-#define MMD_FAILURE()                                       \
-{                                                           \
-    if (device != nullptr)                                  \
-    {                                                       \
-        MOS_Delete(device);                                 \
-    }                                                       \
-    return MOS_STATUS_NO_SPACE;                             \
-}
     MHW_FUNCTION_ENTER;
 
+    if (mhwInterfaces->m_miInterface == nullptr || mhwInterfaces->m_veboxInterface == nullptr)
+    {
+        if (osInterface != nullptr)
+        {
+            if (osInterface->pfnDestroy)
+            {
+                osInterface->pfnDestroy(osInterface, false);
+            }
+            MOS_FreeMemory(osInterface);
+        }
+        return MOS_STATUS_NULL_POINTER;
+    }
+
     Mmd *device = nullptr;
-
-    if (mhwInterfaces->m_miInterface == nullptr)
-    {
-        MMD_FAILURE();
-    }
-
-    if (mhwInterfaces->m_veboxInterface == nullptr)
-    {
-        MMD_FAILURE();
-    }
-
     device = MOS_New(Mmd);
 
     if (device == nullptr)
     {
-        MMD_FAILURE();
+        if (osInterface != nullptr)
+        {
+            if (osInterface->pfnDestroy)
+            {
+                osInterface->pfnDestroy(osInterface, false);
+            }
+            MOS_FreeMemory(osInterface);
+        }
+        return MOS_STATUS_NO_SPACE;
     }
 
+    // transfer ownership of osinterface to device. device will have exclusive ownership of osinterface and free it.
     if (device->Initialize(
         osInterface,
         mhwInterfaces->m_cpInterface,
@@ -317,7 +320,8 @@ MOS_STATUS MmdDeviceXe_Hpm::Initialize(
         mhwInterfaces->m_cpInterface = nullptr;
         mhwInterfaces->m_miInterface = nullptr;
         mhwInterfaces->m_veboxInterface = nullptr;
-        MMD_FAILURE();
+        MOS_Delete(device);
+        return MOS_STATUS_UNKNOWN;
     }
 
     m_mmdDevice = device;
@@ -561,13 +565,21 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
     CodechalDebugInterface *debugInterface = nullptr;
     CodechalHwInterfaceNext    *hwInterface_next    = nullptr;
     
+    auto release_func = [&]() 
+    {
+        MOS_Delete(hwInterface);
+#if USE_CODECHAL_DEBUG_TOOL
+        MOS_Delete(debugInterface);
+#endif  // USE_CODECHAL_DEBUG_TOOL
+    };
+
     if (CodecHalIsDecode(CodecFunction))
     {
         if(osInterface->bHcpDecScalabilityMode == MOS_SCALABILITY_ENABLE_MODE_USER_FORCE)
         {
             disableScalability = false;
         }
-        CreateCodecHalInterface(mhwInterfaces, hwInterface, debugInterface, osInterface, CodecFunction, disableScalability);
+        CODECHAL_PUBLIC_CHK_STATUS_RETURN(CreateCodecHalInterface(mhwInterfaces, hwInterface, debugInterface, osInterface, CodecFunction, disableScalability));
 
     #ifdef _MPEG2_DECODE_SUPPORTED
         if (info->Mode == CODECHAL_DECODE_MODE_MPEG2IDCT ||
@@ -629,18 +641,13 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
     #endif
         {
             CODECHAL_PUBLIC_ASSERTMESSAGE("Decode mode requested invalid!");
-            return MOS_STATUS_INVALID_PARAMETER;
+            CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
         }
 
         if (m_codechalDevice == nullptr)
         {
-            MOS_Delete(hwInterface);
-            mhwInterfaces->SetDestroyState(true);
-#if USE_CODECHAL_DEBUG_TOOL
-            MOS_Delete(debugInterface);
-#endif
             CODECHAL_PUBLIC_ASSERTMESSAGE("Decoder device creation failed!");
-            return MOS_STATUS_NO_SPACE;
+            CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_NO_SPACE, release_func);
         }
     }
     else if (CodecHalIsEncode(CodecFunction))
@@ -653,12 +660,21 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
             return stmt;                           \
         }
 
+        auto release_func_next = [&]() 
+        {
+            MOS_Delete(mhwInterfacesNext);
+            MOS_Delete(hwInterface_next);
+#if USE_CODECHAL_DEBUG_TOOL
+            MOS_Delete(debugInterface);
+#endif  // USE_CODECHAL_DEBUG_TOOL
+        };
+
         CodechalEncoderState *encoder = nullptr;
 
 #if defined (_AVC_ENCODE_VDENC_SUPPORTED)
         if (info->Mode == CODECHAL_ENCODE_MODE_AVC)
         {
-            CreateCodecHalInterface(mhwInterfaces, hwInterface, debugInterface, osInterface, CodecFunction, disableScalability);
+            CODECHAL_PUBLIC_CHK_STATUS_RETURN(CreateCodecHalInterface(mhwInterfaces, hwInterface, debugInterface, osInterface, CodecFunction, disableScalability));
 
             if (CodecHalUsesVdencEngine(info->CodecFunction))
             {
@@ -667,12 +683,12 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
             else
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Encode allocation failed, AVC VME Encoder is not supported, please use AVC LowPower Encoder instead!");
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
             if (encoder == nullptr)
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Encode state creation failed!");
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
             else
             {
@@ -684,13 +700,13 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
 #ifdef _JPEG_ENCODE_SUPPORTED
         if (info->Mode == CODECHAL_ENCODE_MODE_JPEG)
         {
-            CreateCodecHalInterface(mhwInterfaces, hwInterface, debugInterface, osInterface, CodecFunction, disableScalability);
+            CODECHAL_PUBLIC_CHK_STATUS_RETURN(CreateCodecHalInterface(mhwInterfaces, hwInterface, debugInterface, osInterface, CodecFunction, disableScalability));
 
             encoder = MOS_New(Encode::Jpeg, hwInterface, debugInterface, info);
             if (encoder == nullptr)
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Encode state creation failed!");
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
             else
             {
@@ -703,20 +719,20 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
         if (info->Mode == CODECHAL_ENCODE_MODE_MPEG2)
         {
             CODECHAL_PUBLIC_ASSERTMESSAGE("Encode allocation failed, MPEG2 Encoder is not supported!");
-            return MOS_STATUS_INVALID_PARAMETER;
+            CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
         }
         else
 #ifdef _VP9_ENCODE_VDENC_SUPPORTED
         if (info->Mode == CODECHAL_ENCODE_MODE_VP9)
         {
-            CreateCodecHalInterface(mhwInterfaces, hwInterface, debugInterface, osInterface, CodecFunction, disableScalability);
+            CODECHAL_PUBLIC_CHK_STATUS_RETURN(CreateCodecHalInterface(mhwInterfaces, hwInterface, debugInterface, osInterface, CodecFunction, disableScalability));
 
             encoder = MOS_New(Encode::Vp9, hwInterface, debugInterface, info);
 
             if (encoder == nullptr)
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Encode state creation failed!");
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
             else
             {
@@ -728,16 +744,14 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
 #if defined (_AV1_ENCODE_VDENC_SUPPORTED)
         if (info->Mode == codechalEncodeModeAv1)
         {
-            CreateCodecHalInterface(mhwInterfaces, mhwInterfacesNext, hwInterface_next, debugInterface, osInterface, CodecFunction, disableScalability);
+            CODECHAL_PUBLIC_CHK_STATUS_RETURN(CreateCodecHalInterface(mhwInterfaces, mhwInterfacesNext, hwInterface_next, debugInterface, osInterface, CodecFunction, disableScalability));
 
             if (CodecHalUsesVdencEngine(info->CodecFunction))
             {
                 m_codechalDevice = MOS_New(Encode::Av1Vdenc, hwInterface_next, debugInterface);
-                MOS_Delete(hwInterface);
-                hwInterface = nullptr;
                 if (m_codechalDevice == nullptr)
                 {
-                    RETURN_STATUS_WITH_DELETE(MOS_STATUS_NULL_POINTER);
+                    CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_NULL_POINTER, release_func_next);
                 }
                 else
                 {
@@ -746,7 +760,7 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
             }
             else
             {
-                RETURN_STATUS_WITH_DELETE(MOS_STATUS_INVALID_PARAMETER);
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func_next);
             }
         }
         else
@@ -754,7 +768,7 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
 #if defined (_HEVC_ENCODE_VDENC_SUPPORTED)
         if (info->Mode == CODECHAL_ENCODE_MODE_HEVC)
         {
-            CreateCodecHalInterface(mhwInterfaces, mhwInterfacesNext, hwInterface_next, debugInterface, osInterface, CodecFunction, disableScalability);
+            CODECHAL_PUBLIC_CHK_STATUS_RETURN(CreateCodecHalInterface(mhwInterfaces, mhwInterfacesNext, hwInterface_next, debugInterface, osInterface, CodecFunction, disableScalability));
 
             if (CodecHalUsesVdencEngine(info->CodecFunction))
             {
@@ -762,7 +776,7 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
                 if (m_codechalDevice == nullptr)
                 {
                     CODECHAL_PUBLIC_ASSERTMESSAGE("Encode state creation failed!");
-                    RETURN_STATUS_WITH_DELETE(MOS_STATUS_INVALID_PARAMETER);
+                    CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func_next);
                 }
                 RETURN_STATUS_WITH_DELETE(MOS_STATUS_SUCCESS);
             }
@@ -771,7 +785,7 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
 #endif
         {
             CODECHAL_PUBLIC_ASSERTMESSAGE("Unsupported encode function requested.");
-            return MOS_STATUS_INVALID_PARAMETER;
+            CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
         }
 
         if (mhwInterfacesNext != nullptr)
@@ -782,7 +796,7 @@ MOS_STATUS CodechalInterfacesXe_Hpm::Initialize(
     else
     {
         CODECHAL_PUBLIC_ASSERTMESSAGE("Unsupported codec function requested.");
-        return MOS_STATUS_INVALID_PARAMETER;
+        CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
     }
 
     return MOS_STATUS_SUCCESS;
@@ -835,14 +849,12 @@ MOS_STATUS CodechalInterfacesNextXe_Hpm::Initialize(
     if (debugInterface == nullptr)
     {
         MOS_Delete(hwInterface);
-        mhwInterfaces->SetDestroyState(true);
         CODECHAL_PUBLIC_ASSERTMESSAGE("debugInterface is not valid!");
         return MOS_STATUS_NO_SPACE;
     }
     if (debugInterface->Initialize(hwInterface, CodecFunction) != MOS_STATUS_SUCCESS)
     {
         MOS_Delete(hwInterface);
-        mhwInterfaces->SetDestroyState(true);
         MOS_Delete(debugInterface);
         CODECHAL_PUBLIC_ASSERTMESSAGE("Debug interface creation failed!");
         return MOS_STATUS_INVALID_PARAMETER;
@@ -851,10 +863,18 @@ MOS_STATUS CodechalInterfacesNextXe_Hpm::Initialize(
     CodechalDebugInterface *debugInterface = nullptr;
 #endif  // USE_CODECHAL_DEBUG_TOOL
 
+    auto release_func = [&]() 
+    {
+        MOS_Delete(hwInterface);
+#if USE_CODECHAL_DEBUG_TOOL
+        MOS_Delete(debugInterface);
+#endif  // USE_CODECHAL_DEBUG_TOOL
+    };
+
     if (CodecHalIsDecode(CodecFunction))
     {
         CODECHAL_PUBLIC_ASSERTMESSAGE("Decode allocation failed, Decoder with CodechalDeviceNext is not supported!");
-        return MOS_STATUS_INVALID_PARAMETER;
+        CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
  
     }
     else if (CodecHalIsEncode(CodecFunction))
@@ -868,26 +888,29 @@ MOS_STATUS CodechalInterfacesNextXe_Hpm::Initialize(
             if (CodecHalUsesVdencEngine(info->CodecFunction))
             {
                 m_codechalDevice = MOS_New(Encode::Av1Vdenc, hwInterface, debugInterface);
-                CODECHAL_PUBLIC_CHK_NULL_RETURN(m_codechalDevice);
+                if (m_codechalDevice == nullptr)
+                {
+                    CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_NULL_POINTER, release_func);
+                }
                 return MOS_STATUS_SUCCESS;
             }
             else
             {
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
         }
         else
 #endif
         {
             CODECHAL_PUBLIC_ASSERTMESSAGE("Unsupported encode function requested.");
-            return MOS_STATUS_INVALID_PARAMETER;
+            CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
         }
 
     }
     else
     {
         CODECHAL_PUBLIC_ASSERTMESSAGE("Unsupported codec function requested.");
-        return MOS_STATUS_INVALID_PARAMETER;
+        CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
     }
 
     return MOS_STATUS_SUCCESS;
@@ -1003,7 +1026,6 @@ MOS_STATUS CodechalInterfacesXe_Hpm::CreateCodecHalInterface(MhwInterfaces *mhwI
     if (pHwInterface->m_hwInterfaceNext == nullptr)
     {
         MOS_Delete(pHwInterface);
-        mhwInterfaces->SetDestroyState(true);
         CODECHAL_PUBLIC_ASSERTMESSAGE("hwInterfaceNext is not valid!");
         return MOS_STATUS_NO_SPACE;
     }
@@ -1016,14 +1038,12 @@ MOS_STATUS CodechalInterfacesXe_Hpm::CreateCodecHalInterface(MhwInterfaces *mhwI
     if (pDebugInterface == nullptr)
     {
         MOS_Delete(pHwInterface);
-        mhwInterfaces->SetDestroyState(true);
         CODECHAL_PUBLIC_ASSERTMESSAGE("debugInterface is not valid!");
         return MOS_STATUS_NO_SPACE;
     }
     if ((pDebugInterface)->Initialize(pHwInterface, CodecFunction) != MOS_STATUS_SUCCESS)
     {
         MOS_Delete(pHwInterface);
-        mhwInterfaces->SetDestroyState(true);
         MOS_Delete(pDebugInterface);
         CODECHAL_PUBLIC_ASSERTMESSAGE("Debug interface creation failed!");
         return MOS_STATUS_INVALID_PARAMETER;
@@ -1078,14 +1098,12 @@ MOS_STATUS CodechalInterfacesXe_Hpm::CreateCodecHalInterface(MhwInterfaces *mhwI
     if (pDebugInterface == nullptr)
     {
         MOS_Delete(pHwInterface);
-        (pMhwInterfacesNext)->SetDestroyState(true);
         CODECHAL_PUBLIC_ASSERTMESSAGE("debugInterface is not valid!");
         return MOS_STATUS_NO_SPACE;
     }
     if ((pDebugInterface)->Initialize(pHwInterface, CodecFunction) != MOS_STATUS_SUCCESS)
     {
         MOS_Delete(pHwInterface);
-        (pMhwInterfacesNext)->SetDestroyState(true);
         MOS_Delete(pDebugInterface);
         CODECHAL_PUBLIC_ASSERTMESSAGE("Debug interface creation failed!");
         return MOS_STATUS_INVALID_PARAMETER;
