@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2018-2020, Intel Corporation
+* Copyright (c) 2018-2022, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -52,7 +52,7 @@ public:
     //! \param  [in] debugInterface
     //!         Pointer to CodechalDebugInterface
     //!
-    HevcPipeline(CodechalHwInterface *hwInterface, CodechalDebugInterface *debugInterface);
+    HevcPipeline(CodechalHwInterfaceNext *hwInterface, CodechalDebugInterface *debugInterface);
 
     virtual ~HevcPipeline() {}
 
@@ -82,6 +82,11 @@ public:
     //!         Point to slice level command buffer if success, else nullptr
     //!
     MHW_BATCH_BUFFER* GetSliceLvlCmdBuffer();
+    //!
+    //! \brief  Declare Regkeys in the scope of hevc decode
+    //! \return MOS_STATUS
+    //!         MOS_STATUS_SUCCESS if success, else fail reason
+    virtual MOS_STATUS InitUserSetting(MediaUserSettingSharedPtr userSettingPtr) override;
 
     DeclareDecodePacketId(hucS2lPacketId);
     DeclareDecodePacketId(hevcLongPacketId);
@@ -146,7 +151,7 @@ protected:
     //! \return MOS_STATUS
     //!         MOS_STATUS_SUCCESS if success, else fail reason
     //!
-    MOS_STATUS InitContexOption(HevcScalabilityPars& scalPars);
+    virtual MOS_STATUS InitContexOption(HevcScalabilityPars& scalPars);
 
     //!
     //! \brief  Initialize Hevc decode mode
@@ -278,13 +283,12 @@ protected:
     //! \return   MOS_STATUS
     //!           MOS_STATUS_SUCCESS if success, else fail reason
     //!
-    MOS_STATUS DumpSecondLevelBatchBuffer();
+    virtual MOS_STATUS DumpSecondLevelBatchBuffer();
 #endif
 
 protected:
     HevcBasicFeature   *m_basicFeature = nullptr; //!< Point to hevc basic feature
 
-    bool                m_shortFormatInUse = false; //!< Indicate it is Short Format
     HevcDecodeMode      m_decodeMode = baseDecodeMode;
 
     BatchBufferArray   *m_secondLevelBBArray = nullptr; //!< Point to second level batch buffer
@@ -294,6 +298,8 @@ protected:
 
     std::vector<DecodePhase *>  m_phaseList;    //!< Phase list
 
+    bool m_allowVirtualNodeReassign = false;    //!< Whether allow virtual node reassign
+
 #if (_DEBUG || _RELEASE_INTERNAL)
     uint32_t m_rtFrameCount = 0; //!< frame count for real tile decoding
     uint32_t m_vtFrameCount = 0; //!< frame count for virtual tile decoding
@@ -302,7 +308,7 @@ protected:
     bool     m_reportHucCriticalError = false; //!< Flag for reporting huc critical error to regkey
 #endif
 
-MEDIA_CLASS_DEFINE_END(HevcPipeline)
+MEDIA_CLASS_DEFINE_END(decode__HevcPipeline)
 };
 
 }
