@@ -61,11 +61,6 @@ DecodePipeline::DecodePipeline(
     {
         MOS_OS_ASSERTMESSAGE("Initialize CodechalOcaDumper failed!");
     }
-
-    CODECHAL_DEBUG_TOOL(
-        DECODE_ASSERT(debugInterface != nullptr);
-        m_debugInterface = debugInterface;
-    );
 }
 
 MOS_STATUS DecodePipeline::CreateStatusReport()
@@ -191,7 +186,7 @@ MOS_STATUS DecodePipeline::Initialize(void *settings)
     m_decodecp = Create_DecodeCpInterface(codecSettings, m_hwInterface->GetCpInterface(), m_hwInterface->GetOsInterface());
     if (m_decodecp)
     {
-        m_decodecp->RegisterParams(codecSettings);
+        DECODE_CHK_STATUS(m_decodecp->RegisterParams(codecSettings));
     }
     DECODE_CHK_STATUS(CreateFeatureManager());
     DECODE_CHK_STATUS(m_featureManager->Init(codecSettings));
@@ -291,7 +286,7 @@ MOS_STATUS DecodePipeline::Prepare(void *params)
     DECODE_CHK_STATUS(m_featureManager->Update(decodeParams));
     if (m_decodecp)
     {
-        m_decodecp->UpdateParams(true);
+        DECODE_CHK_STATUS(m_decodecp->UpdateParams(true));
     }
     DECODE_CHK_STATUS(m_subPacketManager->Prepare());
 
@@ -419,15 +414,20 @@ MOS_STATUS DecodePipeline::DumpDownSamplingParams(DecodeDownSamplingFeature &dow
 }
 #endif
 
-MOS_STATUS DecodePipeline::DumpBitstream(PMOS_RESOURCE pBitstream, uint32_t size, uint32_t offset)
+MOS_STATUS DecodePipeline::DumpBitstream(PMOS_RESOURCE pBitstream, uint32_t size, uint32_t offset, const char* attrName)
 {
     DECODE_FUNC_CALL();
 
     DECODE_CHK_NULL(pBitstream);
 
+    if(attrName == nullptr)
+    {
+        attrName = CodechalDbgAttr::attrDecodeBitstream;
+    }
+
     DECODE_CHK_STATUS(m_debugInterface->DumpBuffer(
         pBitstream, 
-        CodechalDbgAttr::attrDecodeBitstream,
+        attrName,
         "_DEC", 
         size, 
         offset, 
@@ -497,6 +497,11 @@ MOS_STATUS DecodePipeline::DumpOutput(const DecodeStatusReportData& reportData)
             &dstSurface, CodechalDbgAttr::attrDecodeOutputSurface, "DstSurf"));
     }
 
+    if(m_streamout != nullptr)
+    {
+        m_streamout->DumpOutput(reportData);
+    }
+
 #ifdef _DECODE_PROCESSING_SUPPORTED
     DecodeDownSamplingFeature* downSamplingFeature = dynamic_cast<DecodeDownSamplingFeature*>(
         m_featureManager->GetFeature(DecodeFeatureIDs::decodeDownSampling));
