@@ -28,6 +28,8 @@
 #include "encode_utils.h"
 #include "encode_av1_tile.h"
 #include "encode_av1_basic_feature.h"
+#include "encode_av1_vdenc_preenc.h"
+#include "codechal_debug.h"
 
 namespace encode {
 
@@ -56,7 +58,7 @@ MOS_STATUS Av1VdencPipeline::Initialize(void *settings)
     return MOS_STATUS_SUCCESS;
 }
 
-MOS_STATUS Av1VdencPipeline::Uninitialize()
+MOS_STATUS Av1VdencPipeline::Destroy()
 {
     ENCODE_FUNC_CALL();
 
@@ -82,6 +84,12 @@ MOS_STATUS Av1VdencPipeline::UserFeatureReport()
         "VDENC In Use",
         1,
         MediaUserSetting::Group::Sequence);
+
+    ReportUserSettingForDebug(
+        m_userSettingPtr,
+        "Enable Encode VE CtxBasedScheduling",
+        MOS_VE_CTXBASEDSCHEDULING_SUPPORTED(m_osInterface),
+        MediaUserSetting::Group::Sequence);
 #endif
 
     return MOS_STATUS_SUCCESS;
@@ -98,8 +106,81 @@ MOS_STATUS Av1VdencPipeline::Prepare(void *params)
         return MOS_STATUS_INVALID_PARAMETER;
     }
 
+    auto feature = dynamic_cast<Av1BasicFeature *>(m_featureManager->GetFeature(Av1FeatureIDs::basicFeature));
+    ENCODE_CHK_NULL_RETURN(feature);
+    feature->m_dualEncEnable = m_dualEncEnable;
+
     ENCODE_CHK_STATUS_RETURN(Av1Pipeline::Prepare(params));
 
+    uint16_t numTileRows = 0;
+    uint16_t numTileColumns = 0;
+    RUN_FEATURE_INTERFACE_RETURN(Av1EncodeTile, Av1FeatureIDs::encodeTile, GetTileRowColumns, numTileRows, numTileColumns);
+
+    ENCODE_CHK_STATUS_RETURN(SwitchContext(feature->m_outputChromaFormat, numTileRows, numTileColumns));
+
+    EncoderStatusParameters inputParameters = {};
+
+    ENCODE_CHK_STATUS_RETURN(FillStatusReportParameters(&inputParameters, encodeParams));
+
+    ENCODE_CHK_STATUS_RETURN(m_statusReport->Init(&inputParameters));
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS Av1VdencPipeline::ResetParams()
+{
+    ENCODE_FUNC_CALL();
+
+    m_currRecycledBufIdx = (m_currRecycledBufIdx + 1) % CODECHAL_ENCODE_RECYCLED_BUFFER_NUM;
+
+    if (m_currRecycledBufIdx == 0)
+    {
+        MOS_ZeroMemory(m_recycledBufStatusNum, sizeof(m_recycledBufStatusNum));
+    }
+
+    auto feature = dynamic_cast<EncodeBasicFeature*>(m_featureManager->GetFeature(Av1FeatureIDs::basicFeature));
+    ENCODE_CHK_NULL_RETURN(feature);
+
+    // Only update user features for first frame.
+    if (feature->m_frameNum == 0)
+    {
+        ENCODE_CHK_STATUS_RETURN(UserFeatureReport());
+    }
+
+    feature->m_frameNum++;
+
+    ENCODE_CHK_STATUS_RETURN(m_statusReport->Reset());
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS Av1VdencPipeline::GetStatusReport(void *status, uint16_t numStatus)
+{
+    ENCODE_FUNC_CALL();
+    ENCODE_CHK_STATUS_RETURN(m_statusReport->GetReport(numStatus, status));
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS Av1VdencPipeline::Execute()
+{
+    ENCODE_FUNC_CALL();
+
+    ENCODE_CHK_STATUS_RETURN(ActivateVdencVideoPackets());
+    ENCODE_CHK_STATUS_RETURN(ExecuteActivePackets());
+
+    ENCODE_CHK_STATUS_RETURN(ResetParams());
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS Av1VdencPipeline::HuCCheckAndInit()
+{
+    ENCODE_FUNC_CALL();
+
+    bool immediateSubmit = !m_singleTaskPhaseSupported;
+    ENCODE_CHK_STATUS_RETURN(ActivatePacket(Av1HucBrcInit, immediateSubmit, 0, 0));
+
     return MOS_STATUS_SUCCESS;
 }
 
@@ -117,9 +198,23 @@ MOS_STATUS Av1VdencPipeline::ActivateVdencVideoPackets()
     auto brcFeature = dynamic_cast<Av1Brc*>(m_featureManager->GetFeature(Av1FeatureIDs::av1BrcFeature));
     ENCODE_CHK_NULL_RETURN(brcFeature);
 
+    if (m_preEncEnabled)
+    {
+        ENCODE_CHK_STATUS_RETURN(ActivatePacket(encodePreEncPacket, immediateSubmit, 0, 0));
+#if USE_CODECHAL_DEBUG_TOOL
+        uint32_t encodeMode = 0; 
+        RUN_FEATURE_INTERFACE_RETURN(Av1VdencPreEnc, FeatureIDs::preEncFeature, GetEncodeMode, encodeMode);
+        if (encodeMode == MediaEncodeMode::MANUAL_RES_PRE_ENC || encodeMode == MediaEncodeMode::AUTO_RES_PRE_ENC)
+        {
+            m_activePacketList.back().immediateSubmit = true;
+            return MOS_STATUS_SUCCESS;
+        }
+#endif
+    }
+
     if (brcFeature->IsBRCInitRequired())
     {
-        ENCODE_CHK_STATUS_RETURN(ActivatePacket(Av1HucBrcInit, immediateSubmit, 0, 0));
+        ENCODE_CHK_STATUS_RETURN(HuCCheckAndInit());
     }
 
     for (uint8_t curPass = 0; curPass < GetPassNum(); curPass++)
