@@ -30,8 +30,7 @@
 
 #include "mhw_vebox_itf.h"
 #include "mhw_impl.h"
-#include "hal_oca_interface.h"
-#include "mhw_mi_g12_X.h"
+#include "hal_oca_interface_next.h"
 #include "mos_solo_generic.h"
 
 #ifdef IGFX_VEBOX_INTERFACE_EXT_SUPPORT
@@ -83,15 +82,14 @@ public:
         }
 
 #if (_DEBUG || _RELEASE_INTERNAL)
-        MOS_USER_FEATURE_VALUE_DATA UserFeatureData;
-        // read the "Vebox Split Ratio" user feature
-        MOS_ZeroMemory(&UserFeatureData, sizeof(UserFeatureData));
-        MOS_UserFeature_ReadValue_ID(
-            nullptr,
-            __MEDIA_USER_FEATURE_VALUE_VEBOX_SPLIT_RATIO_ID,
-            &UserFeatureData,
-            osItf->pOsContext);
-        m_veboxSplitRatio = UserFeatureData.u32Data;
+        if (m_userSettingPtr != nullptr)
+        {
+            ReadUserSettingForDebug(
+                m_userSettingPtr,
+                m_veboxSplitRatio,
+                __MEDIA_USER_FEATURE_VALUE_VEBOX_SPLIT_RATIO,
+                MediaUserSetting::Group::Device);
+        }
 #endif
     };
 
@@ -122,8 +120,34 @@ public:
         return tileMode;
     }
 
+    MOS_STATUS UpdateVeboxSync() override
+    {
+        PMHW_VEBOX_HEAP          pVeboxHeap;
+        MOS_STATUS               eStatus = MOS_STATUS_SUCCESS;
+        PMOS_INTERFACE           pOsInterface;
+
+        MHW_FUNCTION_ENTER;
+
+        MHW_CHK_NULL_RETURN(this->m_osItf);
+        MHW_CHK_NULL_RETURN(m_veboxHeap);
+
+        pVeboxHeap      = m_veboxHeap;
+        pOsInterface    = this->m_osItf;
+
+        // If KMD frame tracking is on, the dwSyncTag has been set to gpu status tag
+        // in Mhw_VeboxInterface_AssignVeboxState(). dwNextTag is not used anymore.
+        if (!pOsInterface->bEnableKmdMediaFrameTracking)
+        {
+            pVeboxHeap->pStates[pVeboxHeap->uiCurState].dwSyncTag =
+                pVeboxHeap->dwNextTag++;
+        }
+        pVeboxHeap->pStates[pVeboxHeap->uiCurState].bBusy = true;
+
+        return eStatus;
+    }
+
     MOS_STATUS GetVeboxHeapInfo(
-        const MHW_VEBOX_HEAP** ppVeboxHeap)
+        const MHW_VEBOX_HEAP** ppVeboxHeap) override
     {
         MOS_STATUS               eStatus = MOS_STATUS_SUCCESS;
 
@@ -135,14 +159,27 @@ public:
         return eStatus;
     }
 
-    uint32_t GetVeboxNumInstances()
+    MOS_STATUS SetVeboxHeapStateIndex(
+        uint32_t index) override
+    {
+        MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
+
+        MHW_FUNCTION_ENTER;
+        MHW_CHK_NULL_RETURN(m_veboxHeap);
+
+        m_veboxHeap->uiCurState = index;
+
+        return eStatus;
+    }
+
+    uint32_t GetVeboxNumInstances() override
     {
         MHW_FUNCTION_ENTER;
 
         return m_veboxSettings.uiNumInstances;
     }
 
-    MOS_STATUS DestroyHeap()
+    MOS_STATUS DestroyHeap() override
     {
         PMOS_INTERFACE       pOsInterface;
         MOS_STATUS           eStatus = MOS_STATUS_SUCCESS;
@@ -181,7 +218,7 @@ public:
         return eStatus;
     }
 
-    MOS_STATUS CreateHeap()
+    MOS_STATUS CreateHeap() override
     {
         MOS_STATUS              eStatus;
         uint8_t* pMem;
@@ -277,9 +314,8 @@ public:
         {
             // Use device memory for vebox heap kernel resource, as no cpu access on it.
             AllocParams.dwMemType = MOS_MEMPOOL_DEVICEMEMORY;
-            AllocParams.Flags.bNotLockable = 1;
         }
-
+        AllocParams.Flags.bNotLockable = 1;
         MHW_CHK_STATUS_RETURN(this->m_osItf->pfnAllocateResource(
             this->m_osItf,
             &AllocParams,
@@ -309,7 +345,7 @@ public:
         return eStatus;
     }
 
-    MOS_STATUS SetgnLumaWgts(uint32_t lumaStadTh, uint32_t TGNEThCnt, bool tGNEEnable)
+    MOS_STATUS SetgnLumaWgts(uint32_t lumaStadTh, uint32_t TGNEThCnt, bool tGNEEnable) override
     {
         dw4X4TGNEThCnt = TGNEThCnt;
         bTGNEEnable    = tGNEEnable;
@@ -318,13 +354,37 @@ public:
         return MOS_STATUS_SUCCESS;
     }
 
-    MOS_STATUS SetgnChromaWgts(uint32_t chromaStadTh)
+    MOS_STATUS SetgnChromaWgts(uint32_t chromaStadTh) override
     {
         dwChromaStadTh = chromaStadTh;
 
         return MOS_STATUS_SUCCESS;
     }
 
+    MOS_STATUS SetgnHVSParams(
+        bool tGNEEnable, uint32_t lumaStadTh, uint32_t chromaStadTh, 
+        uint32_t tGNEThCnt, uint32_t historyInit, bool fallBack) override
+    {
+        dw4X4TGNEThCnt = tGNEThCnt;
+        bTGNEEnable    = tGNEEnable;
+        dwLumaStadTh   = lumaStadTh;
+        dwChromaStadTh = chromaStadTh;
+        bHVSfallback   = fallBack;
+        dwHistoryInit  = historyInit;
+
+        return MOS_STATUS_SUCCESS;
+    }
+
+    MOS_STATUS SetgnHVSMode(bool hVSAutoBdrate, bool hVSAutoSubjective, uint32_t bSDThreshold) override
+    {
+        bHVSAutoBdrateEnable     = hVSAutoBdrate;
+        bHVSAutoSubjectiveEnable = hVSAutoSubjective;
+        dwBSDThreshold           = bSDThreshold;
+
+        return MOS_STATUS_SUCCESS;
+    }
+
+
     void RefreshVeboxSync()
     {
         MHW_VEBOX_HEAP              *pVeboxHeap;
@@ -369,7 +429,8 @@ public:
             // The condition below is valid when sync tag wraps from 2^32-1 to 0
             if (((int32_t)(dwCurrentTag - pCurInstance->dwSyncTag) >= 0) ||
                 NullRenderingFlags.VPGobal ||
-                NullRenderingFlags.VPDnDi)
+                NullRenderingFlags.VPDnDi  ||
+                this->m_osItf->bNullHwIsEnabled)
             {
                 pCurInstance->bBusy = false;
             }
@@ -385,25 +446,37 @@ public:
 
     MOS_STATUS SetVeboxSurfaceControlBits(
         PMHW_VEBOX_SURFACE_CNTL_PARAMS pVeboxSurfCntlParams,
-        uint32_t* pSurfCtrlBits)
+        uint32_t* pSurfCtrlBits) override
     {
         return MOS_STATUS_SUCCESS;
     }
 
     MOS_STATUS SetVeboxDndiState(
-        PMHW_VEBOX_DNDI_PARAMS pVeboxDndiParams)
+        PMHW_VEBOX_DNDI_PARAMS pVeboxDndiParams) override
     {
         return MOS_STATUS_SUCCESS;
     }
 
     MOS_STATUS SetVeboxIecpState(
-        PMHW_VEBOX_IECP_PARAMS pVeboxIecpParams)
+        PMHW_VEBOX_IECP_PARAMS pVeboxIecpParams) override
+    {
+        return MOS_STATUS_SUCCESS;
+    }
+
+    MOS_STATUS SetDisableHistogram(
+        PMHW_VEBOX_IECP_PARAMS pVeboxIecpParams) override
+    {
+        return MOS_STATUS_SUCCESS;
+    }
+
+    MOS_STATUS SetAlphaFromStateSelect(
+        PMHW_VEBOX_IECP_PARAMS pVeboxIecpParams) override
     {
         return MOS_STATUS_SUCCESS;
     }
 
     MOS_STATUS SetVeboxLaceColorParams(
-        MHW_LACE_COLOR_CORRECTION *pLaceColorParams)
+        MHW_LACE_COLOR_CORRECTION *pLaceColorParams) override
     {
         MHW_CHK_NULL_RETURN(pLaceColorParams);
         MOS_SecureMemcpy(&m_laceColorCorrection, sizeof(MHW_LACE_COLOR_CORRECTION), pLaceColorParams, sizeof(MHW_LACE_COLOR_CORRECTION));
@@ -412,7 +485,7 @@ public:
     }
 
     MOS_STATUS SetVeboxChromaParams(
-        MHW_VEBOX_CHROMA_PARAMS* chromaParams)
+        MHW_VEBOX_CHROMA_PARAMS* chromaParams) override
     {
         MHW_CHK_NULL_RETURN(chromaParams);
         MOS_SecureMemcpy(&m_chromaParams, sizeof(MHW_VEBOX_CHROMA_PARAMS), chromaParams, sizeof(MHW_VEBOX_CHROMA_PARAMS));
@@ -420,7 +493,7 @@ public:
         return MOS_STATUS_SUCCESS;
     }
 
-    MOS_STATUS AssignVeboxState()
+    MOS_STATUS AssignVeboxState() override
     {
         uint32_t                dwWaitMs, dwWaitTag;
         MOS_STATUS              eStatus = MOS_STATUS_SUCCESS;
@@ -504,36 +577,6 @@ public:
         return eStatus;
     }
 
-    MOS_STATUS setVeboxPrologCmd(
-        PMHW_MI_INTERFACE   mhwMiInterface,
-        PMOS_COMMAND_BUFFER CmdBuffer)
-    {
-        MOS_STATUS                            eStatus = MOS_STATUS_SUCCESS;
-        uint64_t                              auxTableBaseAddr = 0;
-
-        MHW_CHK_NULL_RETURN(mhwMiInterface);
-        MHW_CHK_NULL_RETURN(CmdBuffer);
-        MHW_CHK_NULL_RETURN(this->m_osItf);
-
-        auxTableBaseAddr = this->m_osItf->pfnGetAuxTableBaseAddr(this->m_osItf);
-
-        if (auxTableBaseAddr)
-        {
-            MHW_MI_LOAD_REGISTER_IMM_PARAMS lriParams;
-            MOS_ZeroMemory(&lriParams, sizeof(MHW_MI_LOAD_REGISTER_IMM_PARAMS));
-
-            lriParams.dwRegister = MhwMiInterfaceG12::m_mmioVe0AuxTableBaseLow;
-            lriParams.dwData = (auxTableBaseAddr & 0xffffffff);
-            MHW_CHK_STATUS_RETURN(mhwMiInterface->AddMiLoadRegisterImmCmd(CmdBuffer, &lriParams));
-
-            lriParams.dwRegister = MhwMiInterfaceG12::m_mmioVe0AuxTableBaseHigh;
-            lriParams.dwData = ((auxTableBaseAddr >> 32) & 0xffffffff);
-            MHW_CHK_STATUS_RETURN(mhwMiInterface->AddMiLoadRegisterImmCmd(CmdBuffer, &lriParams));
-        }
-
-        return eStatus;
-    }
-
   MOS_STATUS AdjustBoundary(
         PMHW_VEBOX_SURFACE_PARAMS pCurrSurf,
         uint32_t *pdwSurfaceWidth,
@@ -603,6 +646,8 @@ public:
                 (uint32_t)pCurrSurf->rcSrc.right,
                 *pdwSurfaceHeight,
                 *pdwSurfaceWidth);
+            MT_LOG5(MT_VP_MHW_VE_ADJUST_SURFPARAM, MT_NORMAL, MT_VP_RENDER_VE_CROPPING, 1, MT_RECT_BOTTOM, pCurrSurf->rcSrc.bottom, 
+                MT_RECT_RIGHT, pCurrSurf->rcSrc.right, MT_SURF_HEIGHT, *pdwSurfaceHeight, MT_SURF_WIDTH, *pdwSurfaceWidth);
         }
         else
         {
@@ -641,7 +686,7 @@ public:
     MOS_STATUS SetVeboxIndex(
         uint32_t dwVeboxIndex,
         uint32_t dwVeboxCount,
-        uint32_t dwUsingSFC)
+        uint32_t dwUsingSFC) override
     {
         MOS_STATUS      eStatus = MOS_STATUS_SUCCESS;
 
@@ -660,19 +705,19 @@ public:
         if (isCmBuffer)
         {
             char ocaLog[] = "Vebox indirect state use CmBuffer";
-            HalOcaInterface::TraceMessage(cmdBuffer, mosContext, ocaLog, sizeof(ocaLog));
+            HalOcaInterfaceNext::TraceMessage(cmdBuffer, (MOS_CONTEXT_HANDLE)&mosContext, ocaLog, sizeof(ocaLog));
         }
         else
         {
             if (useVeboxHeapKernelResource)
             {
                 char ocaLog[] = "Vebox indirect state use KernelResource";
-                HalOcaInterface::TraceMessage(cmdBuffer, mosContext, ocaLog, sizeof(ocaLog));
+                HalOcaInterfaceNext::TraceMessage(cmdBuffer, (MOS_CONTEXT_HANDLE)&mosContext, ocaLog, sizeof(ocaLog));
             }
             else
             {
                 char ocaLog[] = "Vebox indirect state use DriverResource";
-                HalOcaInterface::TraceMessage(cmdBuffer, mosContext, ocaLog, sizeof(ocaLog));
+                HalOcaInterfaceNext::TraceMessage(cmdBuffer, (MOS_CONTEXT_HANDLE)&mosContext, ocaLog, sizeof(ocaLog));
             }
         }
         return MOS_STATUS_SUCCESS;
@@ -693,7 +738,7 @@ public:
     MOS_STATUS CreateGpuContext(
         PMOS_INTERFACE  pOsInterface,
         MOS_GPU_CONTEXT VeboxGpuContext,
-        MOS_GPU_NODE    VeboxGpuNode)
+        MOS_GPU_NODE    VeboxGpuNode) override
     {
         MEDIA_FEATURE_TABLE *skuTable;
         MOS_STATUS          eStatus = MOS_STATUS_SUCCESS;
@@ -825,7 +870,7 @@ public:
         PMHW_VEBOX_SURFACE_PARAMS pcurrSurf,
         uint32_t *                pdwSurfaceWidth,
         uint32_t *                pdwSurfaceHeight,
-        bool                      bDIEnable)
+        bool                      bDIEnable) override
     {
         MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
 
@@ -844,7 +889,7 @@ public:
     }
 
     MOS_STATUS FindVeboxGpuNodeToUse(
-        PMHW_VEBOX_GPUNODE_LIMIT pGpuNodeLimit)
+        PMHW_VEBOX_GPUNODE_LIMIT pGpuNodeLimit) override
     {
         MOS_GPU_NODE VeboxGpuNode = MOS_GPU_NODE_VE;
         MOS_STATUS   eStatus      = MOS_STATUS_SUCCESS;
@@ -887,16 +932,26 @@ public:
 
     MOS_STATUS AddVeboxSurfaces(
         PMOS_COMMAND_BUFFER                pCmdBufferInUse,
-        PMHW_VEBOX_SURFACE_STATE_CMD_PARAMS pVeboxSurfaceStateCmdParams)
+        PMHW_VEBOX_SURFACE_STATE_CMD_PARAMS pVeboxSurfaceStateCmdParams) override
     {
         return MOS_STATUS_SUCCESS;
     }
 
-    bool IsVeboxScalabilitywith4K()
+    bool IsVeboxScalabilitywith4K() override
     {
         return m_veboxScalabilitywith4K;
     }
 
+    MOS_STATUS Add1DLutState(void *&surface, PMHW_1DLUT_PARAMS p1DLutParams) override
+    {
+        return MOS_STATUS_SUCCESS;
+    }
+
+    MOS_STATUS AddFP16State(PMHW_FP16_PARAMS pFP16Params) override
+    {
+        return MOS_STATUS_SUCCESS;
+    }
+
     _MHW_SETCMD_OVERRIDE_DECL(VEBOX_SURFACE_STATE)
     {
         _MHW_SETCMD_CALLBASE(VEBOX_SURFACE_STATE);
@@ -970,6 +1025,8 @@ protected:
     uint32_t                  dwLumaStadTh                = 3200;
     uint32_t                  dwChromaStadTh              = 1600;
     uint32_t                  dw4X4TGNEThCnt              = 576;
+    uint32_t                  dwHistoryInit               = 32;
+    uint32_t                  dwBSDThreshold              = 480;
     bool                      bTGNEEnable                 = false;
     bool                      bHVSAutoBdrateEnable        = false;
     bool                      bHVSAutoSubjectiveEnable    = false;
@@ -988,6 +1045,7 @@ protected:
 
     int                    m_veboxHeapInUse = 0;
 
+MEDIA_CLASS_DEFINE_END(mhw__vebox__Impl)
 };
 }  // namespace render
 }  // namespace mhw
