@@ -23,6 +23,7 @@
 #define __VP_RENDER_COMMON_H__
 
 #include <stdint.h>
+#include "surface_type.h"
 
 namespace vp
 {
@@ -35,6 +36,7 @@ namespace vp
 #define GENX_TGLLP_CISAID                   12
 #define CM_MAX_ARGS_PER_KERNEL              255   // compiler only supports up to 255 arguments
 #define CM_PAYLOAD_OFFSET                   32    // CM Compiler generates offset by 32 bytes. This need to be subtracted from kernel data offset.
+#define CM_PAYLOAD_OFFSET_LARGE             64
 #define CM_MAX_THREAD_PAYLOAD_SIZE          2016  // 63 GRF
 #define CM_KERNEL_BINARY_PADDING_SIZE       128                                 // Padding after kernel binary to WA page fault issue.
 #define SURFACE_MASK                        0x7
@@ -43,6 +45,8 @@ struct KERNEL_THREAD_SPACE
 {
     uint32_t uWidth;
     uint32_t uHeight;
+    uint32_t uLocalWidth;
+    uint32_t uLocalHeight;
 };
 
 // Need to consistant with compiler
@@ -63,7 +67,27 @@ enum KRN_ARG_KIND
     ARG_KIND_IMPLICIT_LOCALID       = 0x10,
     ARG_KIND_GENERAL_DEPVEC         = 0x20,
     ARG_KIND_SURFACE_2D_SCOREBOARD  = 0x2A,
-    ARG_KIND_GENERAL_DEPCNT         = 0x30
+    ARG_KIND_GENERAL_DEPCNT         = 0x30,
+
+    //For L0 used only
+    ARG_KIND_INLINE                 = 0xa00
+};
+
+enum KRN_ARG_ADDRESSMODE
+{
+    AddressingModeStateful = 0,
+    AddressingModeStateless,
+    AddressingModeBindless,
+    AddressIngModeMax
+};
+
+enum IMPLICIT_ARG_TYPE
+{
+    ValueType = 0,
+    IndirectDataPtr,
+    ScratchPtr,
+    SamplerStateBasePtr,
+    SurfaceStateBasePtr
 };
 
 struct KRN_ARG
@@ -73,22 +97,56 @@ struct KRN_ARG
     void*                  pData;
     uint32_t               uSize;            // size of arg in byte
     KRN_ARG_KIND           eArgKind;
+    bool                   isOutput;
+    KRN_ARG_ADDRESSMODE    addressMode;
+    IMPLICIT_ARG_TYPE      implicitArgType;
+};
+
+//for L0 use only
+struct KRN_BTI
+{
+    uint32_t               uIndex;
+    uint32_t               uBTI;
+};
+
+//for L0 use only
+struct KRN_EXECUTE_ENV
+{
+    uint32_t uBarrierCount;
+    bool     bDisableMidThreadPreemption;
+    uint32_t uGrfCount;
+    bool     bHasGlobalAtomics;
+    bool     bHasNoStatelessWrite;
+    uint32_t uInlineDataPayloadSize;
+    uint32_t uOffsetToSkipPerThreadDataLoad;
+    uint32_t uSimdSize;
+    uint32_t uSubgroupIndependentForwardProgress;
+    uint32_t uEuThreadCount;
+    bool     bHasFenceForImageAccess;
+    bool     bHasSample;
+    bool     bHas4GBBuffers;
+    uint8_t  uiWorkGroupWalkOrderDimensions[3];
+    uint64_t uiPrivateSize;
+    uint32_t uiSlmSize;
 };
 
 using SurfaceIndex = uint32_t;
 using SamplerIndex = uint32_t;
 using KernelIndex  = uint32_t;              // index of current kernel in KERNEL_PARAMS_LIST
 
+typedef struct _SURFACE_PARAMS
+{
+    SurfaceType surfType;
+    bool        isOutput;
+    bool        needVerticalStirde;
+    bool        combineChannelY;
+} SURFACE_PARAMS, *PSURFACE_PARAMS;
+using KERNEL_ARG_INDEX_SURFACE_MAP = std::map<uint32_t, SURFACE_PARAMS>;
+
 enum KERNEL_SUBMISSION_MODE
 {
-    MULTI_KERNELS_WITH_MULTI_MEDIA_STATES = 0,
-    MULTI_KERNELS_WITH_ONE_MEDIA_STATE
-};
-
-enum KERNEL_BINDINGTABLE_MODE
-{
-    MULTI_KERNELS_WITH_MULTI_BINDINGTABLES = 0,
-    MULTI_KERNELS_WITH_ONE_BINDINGTABLE
+    SINGLE_KERNEL_ONLY = 0,
+    MULTI_KERNELS_SINGLE_MEDIA_STATE,
 };
 
 typedef struct _VP_RENDER_CACHE_CNTL
