@@ -692,6 +692,11 @@ MOS_STATUS VpProcampReuse::UpdatePacket(SwFilter *filter, VpCmdPacket *packet)
 MOS_STATUS VpProcampReuse::UpdateFeatureParams(FeatureParamProcamp &params)
 {
     m_params = params;
+    if (params.procampParams)
+    {
+        m_procampParams        = *params.procampParams;
+        m_params.procampParams = &m_procampParams;
+    }
     return MOS_STATUS_SUCCESS;
 }
 
@@ -907,6 +912,12 @@ MOS_STATUS VpPacketReuseManager::PreparePacketPipeReuse(SwFilterPipe *&swFilterP
         auto scalingreuse = m_features.find(FeatureTypeScaling);
         auto cscreuse     = m_features.find(FeatureTypeCsc);
         auto rotreuse     = m_features.find(FeatureTypeRotMir);
+        if (scalingreuse == m_features.end() ||
+            cscreuse == m_features.end()     ||
+            rotreuse == m_features.end())
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_HANDLE);
+        }
 
         for (index = 0; index < m_pipeReused_TeamsPacket.size(); index++)
         {
@@ -1095,6 +1106,13 @@ MOS_STATUS VpPacketReuseManager::UpdatePacketPipeConfig(PacketPipe *&pipe)
         return MOS_STATUS_SUCCESS;
     }
 
+    if (caps.enableSFCLinearOutputByTileConvert)
+    {
+        VP_PUBLIC_NORMALMESSAGE("Not reusable for enableSFCLinearOutputByTileConvert case.");
+        m_reusable = false;
+        return MOS_STATUS_SUCCESS;
+    }
+
     if (m_TeamsPacket && !m_TeamsPacket_reuse)
     {
         auto it = m_pipeReused_TeamsPacket.find(curIndex);
