@@ -366,6 +366,8 @@ protected:
         return MOS_STATUS_SUCCESS;
     }
 
+    virtual MOS_STATUS UpdateFrameTracker();
+
 #if (_DEBUG || _RELEASE_INTERNAL)
     //!
     //! \brief  Report INTER_FRAME_MEMORY_NINJA_START_COUNTER and INTER_FRAME_MEMORY_NINJA_END_COUNTER
@@ -434,6 +436,7 @@ protected:
     MOS_STATUS UpdateVeboxNumberforScalability();
 
     MOS_STATUS ExecuteSingleswFilterPipe(VpSinglePipeContext *singlePipeCtx, SwFilterPipe *&pipe, PacketPipe *pPacketPipe, VpFeatureManagerNext *featureManagerNext);
+    MOS_STATUS UpdateRectForNegtiveDstTopLeft(PVP_PIPELINE_PARAMS params);
 
 protected:
     VP_PARAMS              m_pvpParams              = {};   //!< vp Pipeline params
