@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2020-2021, Intel Corporation
+* Copyright (c) 2020-2024, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -32,6 +32,8 @@
 #include "mhw_utilities.h"
 #include "mhw_mi_cmdpar.h"
 
+#define VP_SEMAPHORE_COUNT 1024
+
 namespace vp
 {
 VpScalabilityMultiPipeNext::VpScalabilityMultiPipeNext(void *hwInterface, MediaContext *mediaContext, uint8_t componentType)
@@ -64,64 +66,47 @@ MOS_STATUS VpScalabilityMultiPipeNext::AllocateSemaphore()
     MOS_LOCK_PARAMS lockFlagsWriteOnly;
     MOS_ZeroMemory(&lockFlagsWriteOnly, sizeof(MOS_LOCK_PARAMS));
     lockFlagsWriteOnly.WriteOnly = 1;
+
     MOS_ALLOC_GFXRES_PARAMS allocParamsForBufferLinear;
     MOS_ZeroMemory(&allocParamsForBufferLinear, sizeof(MOS_ALLOC_GFXRES_PARAMS));
     allocParamsForBufferLinear.TileType = MOS_TILE_LINEAR;
     allocParamsForBufferLinear.Format   = Format_Buffer;
     allocParamsForBufferLinear.Type     = MOS_GFXRES_BUFFER;
-    allocParamsForBufferLinear.dwBytes  = sizeof(uint32_t);
+    allocParamsForBufferLinear.dwBytes  = VP_SEMAPHORE_COUNT * sizeof(uint32_t);
     allocParamsForBufferLinear.pBufName = "Sync All Pipes SemaphoreMemory";
 
-    m_resSemaphoreAllPipes.resize(m_maxCmdBufferSetsNum);
-    for (auto &semaphoreBufferVec : m_resSemaphoreAllPipes)
+    for (uint32_t i = 0; i < 2; i++)
     {
-        semaphoreBufferVec.resize(m_scalabilityOption->GetNumPipe());
-        for (auto &semaphoreBuffer : semaphoreBufferVec)
-        {
-            memset(&semaphoreBuffer, 0, sizeof(MOS_RESOURCE));
-            SCALABILITY_CHK_STATUS_MESSAGE_RETURN(m_osInterface->pfnAllocateResource(
-                                                      m_osInterface,
-                                                      &allocParamsForBufferLinear,
-                                                      &semaphoreBuffer),
-                "Cannot create HW semaphore for scalability all pipes sync.");
-            uint32_t *data = (uint32_t *)m_osInterface->pfnLockResource(
-                m_osInterface,
-                &semaphoreBuffer,
-                &lockFlagsWriteOnly);
-            SCALABILITY_CHK_NULL_RETURN(data);
-            *data = 0;
-            SCALABILITY_CHK_STATUS_RETURN(m_osInterface->pfnUnlockResource(
-                m_osInterface,
-                &semaphoreBuffer));
-        }
-    }
+        memset(&m_resSemaphoreAllPipes[i], 0, sizeof(MOS_RESOURCE));
+        SCALABILITY_CHK_STATUS_MESSAGE_RETURN(m_osInterface->pfnAllocateResource(
+                                                  m_osInterface,
+                                                  &allocParamsForBufferLinear,
+                                                  &m_resSemaphoreAllPipes[i]),
+            "Cannot create HW semaphore for scalability all pipes sync.");
 
-    allocParamsForBufferLinear.pBufName = "Sync One Pipe Wait SemaphoreMemory";
-    m_resSemaphoreOnePipeWait.resize(m_maxCmdBufferSetsNum);
-    for (auto &semaphoreBufferVec : m_resSemaphoreOnePipeWait)
-    {
-        semaphoreBufferVec.resize(m_scalabilityOption->GetNumPipe());
-        for (auto &semaphoreBuffer : semaphoreBufferVec)
-        {
-            memset(&semaphoreBuffer, 0, sizeof(MOS_RESOURCE));
-            SCALABILITY_CHK_STATUS_MESSAGE_RETURN(m_osInterface->pfnAllocateResource(
-                                                      m_osInterface,
-                                                      &allocParamsForBufferLinear,
-                                                      &semaphoreBuffer),
-                "Cannot create HW semaphore for scalability one pipe sync.");
-            uint32_t *data = (uint32_t *)m_osInterface->pfnLockResource(
-                m_osInterface,
-                &semaphoreBuffer,
-                &lockFlagsWriteOnly);
-            SCALABILITY_CHK_NULL_RETURN(data);
-            *data = 0;
-            SCALABILITY_CHK_STATUS_RETURN(m_osInterface->pfnUnlockResource(
-                m_osInterface,
-                &semaphoreBuffer));
-        }
-    }
+        MOS_LOCK_PARAMS lockFlagsWriteOnly;
+        MOS_ZeroMemory(&lockFlagsWriteOnly, sizeof(MOS_LOCK_PARAMS));
+        lockFlagsWriteOnly.WriteOnly = 1;
 
-    m_semaphoreIndex = 0;
+        uint32_t *data = (uint32_t *)m_osInterface->pfnLockResource(
+            m_osInterface,
+            &m_resSemaphoreAllPipes[i],
+            &lockFlagsWriteOnly);
+        SCALABILITY_CHK_NULL_RETURN(data);
+        MOS_ZeroMemory(data, VP_SEMAPHORE_COUNT * sizeof(uint32_t));
+        SCALABILITY_CHK_STATUS_RETURN(m_osInterface->pfnUnlockResource(
+            m_osInterface,
+            &m_resSemaphoreAllPipes[i]));
+    }
+    memset(&m_resSemaphoreOnePipeWait, 0, sizeof(MOS_RESOURCE));
+    SCALABILITY_CHK_STATUS_MESSAGE_RETURN(m_osInterface->pfnAllocateResource(
+                                              m_osInterface,
+                                              &allocParamsForBufferLinear,
+                                              &m_resSemaphoreOnePipeWait),
+        "Cannot create HW semaphore for scalability one pipe sync.");
+
+    m_semaphoreAllPipesIndex = 0;
+    m_semaphoreAllPipesPhase = 0;
 
     return MOS_STATUS_SUCCESS;
 }
@@ -142,7 +127,16 @@ MOS_STATUS VpScalabilityMultiPipeNext::Initialize(const MediaScalabilityOption &
     SCALABILITY_CHK_NULL_RETURN(vpScalabilityOption);
     m_scalabilityOption = vpScalabilityOption;
 
-    m_frameTrackingEnabled = m_osInterface->bEnableKmdMediaFrameTracking ? true : false;
+    if (m_hwInterface->m_bIsMediaSfcInterfaceInUse)
+    {
+        m_frameTrackingEnabled = false;
+        VP_PUBLIC_NORMALMESSAGE("Media Frame Tracking is disabled for Media Sfc Interface enalbed.");
+    }
+    else
+    {
+        m_frameTrackingEnabled = m_osInterface->bEnableKmdMediaFrameTracking ? true : false;
+        VP_PUBLIC_NORMALMESSAGE("Media Frame Tracking flag is %d. Not using Meida Sfc Interface.", m_frameTrackingEnabled);
+    }
 
     //virtual engine init with scalability
     MOS_VIRTUALENGINE_INIT_PARAMS veInitParms;
@@ -152,15 +146,15 @@ MOS_STATUS VpScalabilityMultiPipeNext::Initialize(const MediaScalabilityOption &
     veInitParms.ucMaxNumPipesInUse             = vpScalabilityOption->GetMaxMultiPipeNum();
     veInitParms.ucMaxNumOfSdryCmdBufInOneFrame = veInitParms.ucMaxNumPipesInUse;
 
-    SCALABILITY_CHK_NULL_RETURN(m_osInterface->osStreamState);
-    m_osInterface->osStreamState->component = COMPONENT_VPCommon;
-
-    SCALABILITY_CHK_STATUS_RETURN(MosInterface::CreateVirtualEngineState(
-        m_osInterface->osStreamState, &veInitParms, m_veState));
-    SCALABILITY_CHK_NULL_RETURN(m_veState);
-
-    SCALABILITY_CHK_STATUS_RETURN(MosInterface::GetVeHintParams(m_osInterface->osStreamState, true, &m_veHitParams));
-    SCALABILITY_CHK_NULL_RETURN(m_veHitParams);
+    if (m_osInterface->apoMosEnabled)
+    {
+        SCALABILITY_CHK_NULL_RETURN(m_osInterface->osStreamState);
+        m_osInterface->osStreamState->component = COMPONENT_VPCommon;
+        SCALABILITY_CHK_STATUS_RETURN(m_osInterface->pfnVirtualEngineInit(m_osInterface, &m_veHitParams, veInitParms));
+        m_veState = m_osInterface->osStreamState->virtualEngineInterface;
+        SCALABILITY_CHK_NULL_RETURN(m_veState);
+        SCALABILITY_CHK_NULL_RETURN(m_veHitParams);
+    }
 
     m_pipeNum = m_scalabilityOption->GetNumPipe();
     m_pipeIndexForSubmit = m_pipeNum;
@@ -169,14 +163,16 @@ MOS_STATUS VpScalabilityMultiPipeNext::Initialize(const MediaScalabilityOption &
     SCALABILITY_CHK_NULL_RETURN(gpuCtxCreateOption);
     gpuCtxCreateOption->LRCACount = vpScalabilityOption->GetLRCACount();
     gpuCtxCreateOption->UsingSFC  = vpScalabilityOption->IsUsingSFC();
+    gpuCtxCreateOption->RAMode    = vpScalabilityOption->GetRAMode();
+    gpuCtxCreateOption->ProtectMode = vpScalabilityOption->GetProtectMode();
 
 #if (_DEBUG || _RELEASE_INTERNAL)
     if (m_osInterface->bEnableDbgOvrdInVE)
     {
-        for (uint32_t i = 0; i < MosInterface::GetVeEngineCount(m_osInterface->osStreamState); i++)
+        for (uint32_t i = 0; i < m_osInterface->pfnGetVeEngineCount(m_osInterface->osStreamState); i++)
         {
             gpuCtxCreateOption->EngineInstance[i] =
-                MosInterface::GetEngineLogicId(m_osInterface->osStreamState, i);
+                m_osInterface->pfnGetEngineLogicIdByIdx(m_osInterface->osStreamState, i);
         }
     }
 #endif
@@ -222,20 +218,9 @@ MOS_STATUS VpScalabilityMultiPipeNext::Destroy()
         MOS_Delete(m_scalabilityOption);
     }
 
-    for (auto &semaphoreBufferVec : m_resSemaphoreAllPipes)
-    {
-        for (auto &semaphoreBuffer : semaphoreBufferVec)
-        {
-            m_osInterface->pfnFreeResource(m_osInterface, &semaphoreBuffer);
-        }
-    }
-    for (auto &semaphoreBufferVec : m_resSemaphoreOnePipeWait)
-    {
-        for (auto &semaphoreBuffer : semaphoreBufferVec)
-        {
-            m_osInterface->pfnFreeResource(m_osInterface, &semaphoreBuffer);
-        }
-    }
+    m_osInterface->pfnFreeResource(m_osInterface, &m_resSemaphoreAllPipes[0]);
+    m_osInterface->pfnFreeResource(m_osInterface, &m_resSemaphoreAllPipes[1]);
+    m_osInterface->pfnFreeResource(m_osInterface, &m_resSemaphoreOnePipeWait);
 
     return MOS_STATUS_SUCCESS;
 }
@@ -370,8 +355,8 @@ MOS_STATUS VpScalabilityMultiPipeNext::GetCmdBuffer(PMOS_COMMAND_BUFFER cmdBuffe
         submissionType |= SUBMISSION_TYPE_MULTI_PIPE_FLAGS_LAST_PIPE;
     }
     SCALABILITY_CHK_NULL_RETURN(m_osInterface->osStreamState);
-    SCALABILITY_CHK_STATUS_RETURN(MosInterface::SetVeSubmissionType(
-        m_osInterface->osStreamState, &(m_secondaryCmdBuffers[bufIdx]), submissionType));
+    SCALABILITY_CHK_NULL_RETURN(m_osInterface->osStreamState->virtualEngineInterface);
+    SCALABILITY_CHK_STATUS_RETURN(m_osInterface->osStreamState->virtualEngineInterface->SetSubmissionType(&(m_secondaryCmdBuffers[bufIdx]), submissionType));
 
     *cmdBuffer = m_secondaryCmdBuffers[bufIdx];
     m_secondaryCmdBuffersReturned[bufIdx] = false;
@@ -423,7 +408,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::SetHintParams()
 
     SCALABILITY_FUNCTION_ENTER;
 
-    SCALABILITY_CHK_NULL_RETURN(m_osInterface->osStreamState);
+    SCALABILITY_CHK_NULL_RETURN(m_osInterface);
 
     VpScalabilityOption *vpScalabilityOption = dynamic_cast<VpScalabilityOption *>(m_scalabilityOption);
     SCALABILITY_CHK_NULL_RETURN(vpScalabilityOption);
@@ -434,7 +419,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::SetHintParams()
     veParams.ucScalablePipeNum = m_pipeNum;
     veParams.bScalableMode     = true;
 
-    SCALABILITY_CHK_STATUS_RETURN(MosInterface::SetVeHintParams(m_osInterface->osStreamState, &veParams));
+    SCALABILITY_CHK_STATUS_RETURN(m_osInterface->pfnSetHintParams(m_osInterface, &veParams));
 
     return MOS_STATUS_SUCCESS;
 }
@@ -446,8 +431,9 @@ MOS_STATUS VpScalabilityMultiPipeNext::PopulateHintParams(PMOS_COMMAND_BUFFER cm
     SCALABILITY_FUNCTION_ENTER;
     SCALABILITY_CHK_NULL_RETURN(cmdBuffer);
     SCALABILITY_CHK_NULL_RETURN(m_veHitParams);
+    SCALABILITY_CHK_NULL_RETURN(m_osInterface);
 
-    PMOS_CMD_BUF_ATTRI_VE attriVe  = MosInterface::GetAttributeVeBuffer(cmdBuffer);
+    PMOS_CMD_BUF_ATTRI_VE attriVe  = m_osInterface->pfnGetAttributeVeBuffer(cmdBuffer);
     if (attriVe)
     {
         attriVe->VEngineHintParams     = *(m_veHitParams);
@@ -502,36 +488,82 @@ MOS_STATUS VpScalabilityMultiPipeNext::SyncAllPipes(PMOS_COMMAND_BUFFER cmdBuffe
     SCALABILITY_FUNCTION_ENTER;
     SCALABILITY_CHK_NULL_RETURN(cmdBuffer);
     SCALABILITY_CHK_NULL_RETURN(m_hwInterface);
+    SCALABILITY_ASSERT(m_semaphoreAllPipesIndex < VP_SEMAPHORE_COUNT * 2);
+    SCALABILITY_ASSERT(!Mos_ResourceIsNull(&m_resSemaphoreAllPipes[0]));
+    SCALABILITY_ASSERT(!Mos_ResourceIsNull(&m_resSemaphoreAllPipes[1]));
 
-    SCALABILITY_ASSERT(m_semaphoreIndex < m_resSemaphoreAllPipes.size());
-    auto &semaphoreBufs = m_resSemaphoreAllPipes[m_semaphoreIndex];
-    SCALABILITY_ASSERT(semaphoreBufs.size() >= m_scalabilityOption->GetNumPipe());
+    // use two Semaphore Buffer repeatly to avoid waiting for lock, use the first Semaphore Buffer when m_semaphoreAllPipesIndex < VP_SEMAPHORE_COUNT.
+    // when the m_semaphoreAllPipesIndex >= VP_SEMAPHORE_COUNT, switch to the second Semaphore Buffer
+    uint32_t semaphoreIndex = (m_semaphoreAllPipesIndex >= VP_SEMAPHORE_COUNT) ? 1 : 0;
+
+    if (m_semaphoreAllPipesPhase == 0)
+    {
+        if (m_currentPipe == 0)
+        {
+            std::stringstream ss;
+            ss << "VEBOX0 SCALABILITY. semaphoreIndex 0x" << std::hex << semaphoreIndex << " m_semaphoreAllPipesIndex 0x"
+                << std::hex  << m_semaphoreAllPipesIndex ;
+            std::string ocaMsg = ss.str();
+            HalOcaInterfaceNext::TraceMessage(*cmdBuffer, (MOS_CONTEXT_HANDLE)m_osInterface->pOsContext, ocaMsg.c_str(), ocaMsg.length());
+        }
+        else
+        {
+            std::stringstream ss;
+            ss << "VEBOX1 SCALABILITY. semaphoreIndex 0x" << std::hex << semaphoreIndex << " m_semaphoreAllPipesIndex 0x"
+                << std::hex  << m_semaphoreAllPipesIndex ;
+            std::string ocaMsg = ss.str();
+            HalOcaInterfaceNext::TraceMessage(*cmdBuffer, (MOS_CONTEXT_HANDLE)m_osInterface->pOsContext, ocaMsg.c_str(), ocaMsg.length());
+        }
+
+        for (int32_t i = 0; i < 2 * m_pipeNum; i++)
+        {
+            HalOcaInterfaceNext::OnIndirectState(*cmdBuffer, (MOS_CONTEXT_HANDLE)m_osInterface->pOsContext, &m_resSemaphoreAllPipes[semaphoreIndex], (m_semaphoreAllPipesIndex % VP_SEMAPHORE_COUNT + i) * sizeof(uint32_t), false, sizeof(uint32_t));
+        }
+    }
+
+    // memset the semaphore to avoid the semaphore was not clear during media reset
+    if ((m_semaphoreAllPipesPhase == 0) && (m_currentPipe == 0) && (m_semaphoreAllPipesIndex % VP_SEMAPHORE_COUNT == 0))
+    {
+        MOS_LOCK_PARAMS lockFlagsWriteOnly;
+        MOS_ZeroMemory(&lockFlagsWriteOnly, sizeof(MOS_LOCK_PARAMS));
+        lockFlagsWriteOnly.WriteOnly = 1;
+
+        uint32_t *data = (uint32_t *)m_osInterface->pfnLockResource(
+            m_osInterface,
+            &m_resSemaphoreAllPipes[semaphoreIndex],
+            &lockFlagsWriteOnly);
+        SCALABILITY_CHK_NULL_RETURN(data);
+        MOS_ZeroMemory(data, VP_SEMAPHORE_COUNT * sizeof(uint32_t));
+        SCALABILITY_CHK_STATUS_RETURN(m_osInterface->pfnUnlockResource(
+            m_osInterface,
+            &m_resSemaphoreAllPipes[semaphoreIndex]));
+    }
 
     // Increment all pipe flags
     for (uint32_t i = 0; i < m_pipeNum; i++)
     {
-        if (!Mos_ResourceIsNull(&semaphoreBufs[i]))
+        if (!Mos_ResourceIsNull(&m_resSemaphoreAllPipes[semaphoreIndex]))
         {
             SCALABILITY_CHK_STATUS_RETURN(SendMiAtomicDwordCmd(
-                &semaphoreBufs[i], 1, MHW_MI_ATOMIC_INC, cmdBuffer));
+                &m_resSemaphoreAllPipes[semaphoreIndex], (m_semaphoreAllPipesIndex % VP_SEMAPHORE_COUNT + m_semaphoreAllPipesPhase * m_pipeNum + i) * sizeof(uint32_t), 1, MHW_MI_ATOMIC_INC, cmdBuffer));
         }
     }
 
-    if (!Mos_ResourceIsNull(&semaphoreBufs[m_currentPipe]))
+    if (!Mos_ResourceIsNull(&m_resSemaphoreAllPipes[semaphoreIndex]))
     {
         // Waiting current pipe flag euqal to pipe number which means other pipes are executing
         SCALABILITY_CHK_STATUS_RETURN(SendHwSemaphoreWaitCmd(
-            &semaphoreBufs[m_currentPipe], m_pipeNum, MHW_MI_SAD_EQUAL_SDD, cmdBuffer));
+            &m_resSemaphoreAllPipes[semaphoreIndex], (m_semaphoreAllPipesIndex % VP_SEMAPHORE_COUNT + m_semaphoreAllPipesPhase * m_pipeNum + m_currentPipe) * sizeof(uint32_t), m_pipeNum, MHW_MI_SAD_EQUAL_SDD, cmdBuffer));
 
         // Reset current pipe semaphore
         SCALABILITY_CHK_STATUS_RETURN(AddMiStoreDataImmCmd(
-            &semaphoreBufs[m_currentPipe], cmdBuffer));
+            &m_resSemaphoreAllPipes[semaphoreIndex], (m_semaphoreAllPipesIndex % VP_SEMAPHORE_COUNT + m_semaphoreAllPipesPhase * m_pipeNum + m_currentPipe) * sizeof(uint32_t), cmdBuffer));
     }
 
-    m_semaphoreIndex += m_initSecondaryCmdBufNum;
-    if (m_semaphoreIndex >= m_maxCmdBufferSetsNum)
+    m_semaphoreAllPipesPhase = 1 - m_semaphoreAllPipesPhase;
+    if ((m_semaphoreAllPipesPhase == 0) && (m_currentPipe == 1))
     {
-        m_semaphoreIndex = 0;
+        m_semaphoreAllPipesIndex = (m_semaphoreAllPipesIndex + 2 * m_pipeNum) % (VP_SEMAPHORE_COUNT * 2);
     }
 
     return MOS_STATUS_SUCCESS;
@@ -543,34 +575,32 @@ MOS_STATUS VpScalabilityMultiPipeNext::SyncOnePipeWaitOthers(PMOS_COMMAND_BUFFER
 
     SCALABILITY_FUNCTION_ENTER;
     SCALABILITY_CHK_NULL_RETURN(cmdBuffer);
-
-    SCALABILITY_ASSERT(m_semaphoreIndex < m_resSemaphoreOnePipeWait.size());
-    auto &semaphoreBufs = m_resSemaphoreOnePipeWait[m_semaphoreIndex];
-    SCALABILITY_ASSERT(semaphoreBufs.size() >= m_scalabilityOption->GetNumPipe());
+    SCALABILITY_CHK_NULL_RETURN(m_hwInterface);
+    SCALABILITY_ASSERT(!Mos_ResourceIsNull(&m_resSemaphoreOnePipeWait));
 
     // Send MI_FLUSH command
     SCALABILITY_CHK_STATUS_RETURN(AddMiFlushDwCmd(
-            &semaphoreBufs[m_currentPipe], m_currentPass + 1, cmdBuffer));
+            &m_resSemaphoreOnePipeWait, 0, cmdBuffer));
 
     if (m_currentPipe == pipeIdx)
     {
         // this pipe needs to ensure all other pipes are ready
         for (uint32_t i = 0; i < m_pipeNum; i++)
         {
-            if (!Mos_ResourceIsNull(&semaphoreBufs[i]))
+            if (!Mos_ResourceIsNull(&m_resSemaphoreOnePipeWait))
             {
                 SCALABILITY_CHK_STATUS_RETURN(SendHwSemaphoreWaitCmd(
-                        &semaphoreBufs[i], m_currentPass + 1, MHW_MI_SAD_EQUAL_SDD, cmdBuffer));
+                    &m_resSemaphoreOnePipeWait, i, m_currentPass + 1, MHW_MI_SAD_EQUAL_SDD, cmdBuffer));
             }
         }
 
         // Reset all pipe flags for next frame
         for (uint32_t i = 0; i < m_pipeNum; i++)
         {
-            if (!Mos_ResourceIsNull(&semaphoreBufs[i]))
+            if (!Mos_ResourceIsNull(&m_resSemaphoreOnePipeWait))
             {
                 SCALABILITY_CHK_STATUS_RETURN(SendMiAtomicDwordCmd(
-                    &semaphoreBufs[i], m_currentPass + 1, MHW_MI_ATOMIC_DEC, cmdBuffer));
+                    &m_resSemaphoreOnePipeWait, i, m_currentPass + 1, MHW_MI_ATOMIC_DEC, cmdBuffer));
 
             }
         }
@@ -615,7 +645,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::UpdateState(void *statePars)
     SCALABILITY_FUNCTION_ENTER;
 
     StateParams *vpStatePars = (StateParams *)statePars;
-    if (vpStatePars->currentPipe < 0 || vpStatePars->currentPipe >= m_pipeNum)
+    if (vpStatePars->currentPipe >= m_pipeNum)
     {
         SCALABILITY_ASSERTMESSAGE("UpdateState failed with invalid parameter: currentPipe %d!",
             vpStatePars->currentPipe);
@@ -678,6 +708,8 @@ MOS_STATUS VpScalabilityMultiPipeNext::SendAttrWithFrameTracking(
 //!
 //! \param    [in] semaMem
 //!           Reource of Hw semphore
+//! \param    [in] offset
+//!           offset of semMem
 //! \param    [in] semaData
 //!           Data of Hw semphore
 //! \param    [in] opCode
@@ -690,6 +722,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::SendAttrWithFrameTracking(
 //!
 MOS_STATUS VpScalabilityMultiPipeNext::SendHwSemaphoreWaitCmd(
     PMOS_RESOURCE                             semaMem,
+    uint32_t                                  offset,
     uint32_t                                  semaData,
     MHW_COMMON_MI_SEMAPHORE_COMPARE_OPERATION opCode,
     PMOS_COMMAND_BUFFER                       cmdBuffer)
@@ -704,6 +737,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::SendHwSemaphoreWaitCmd(
     auto &params             = m_miItf->MHW_GETPAR_F(MI_SEMAPHORE_WAIT)();
     params                   = {};
     params.presSemaphoreMem = semaMem;
+    params.dwResourceOffset = offset;
     params.bPollingWaitMode = true;
     params.dwSemaphoreData  = semaData;
     params.CompareOperation = (mhw::mi::MHW_COMMON_MI_SEMAPHORE_COMPARE_OPERATION) opCode;
@@ -718,6 +752,8 @@ MOS_STATUS VpScalabilityMultiPipeNext::SendHwSemaphoreWaitCmd(
 //!
 //! \param    [in] resource
 //!           Reource used in mi atomic dword cmd
+//! \param    [in] offset
+//!           offset of resource
 //! \param    [in] immData
 //!           Immediate data
 //! \param    [in] opCode
@@ -730,6 +766,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::SendHwSemaphoreWaitCmd(
 //!
 MOS_STATUS VpScalabilityMultiPipeNext::SendMiAtomicDwordCmd(
     PMOS_RESOURCE               resource,
+    uint32_t                    offset,
     uint32_t                    immData,
     MHW_COMMON_MI_ATOMIC_OPCODE opCode,
     PMOS_COMMAND_BUFFER         cmdBuffer)
@@ -745,6 +782,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::SendMiAtomicDwordCmd(
     auto &params             = m_miItf->MHW_GETPAR_F(MI_ATOMIC)();
     params                   = {};
     params.pOsResource       = resource;
+    params.dwResourceOffset  = offset;
     params.dwDataSize        = sizeof(uint32_t);
     params.Operation         = (mhw::mi::MHW_COMMON_MI_ATOMIC_OPCODE) opCode;
     params.bInlineData       = true;
@@ -788,7 +826,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::AddMiFlushDwCmd(
         parFlush.pOsResource = semaMem;
         parFlush.dwDataDW1   = semaData + 1;
     }
-    m_miItf->MHW_ADDCMD_F(MI_FLUSH_DW)(cmdBuffer);
+    VP_RENDER_CHK_STATUS_RETURN(m_miItf->MHW_ADDCMD_F(MI_FLUSH_DW)(cmdBuffer));
 
     return eStatus;
 }
@@ -799,6 +837,8 @@ MOS_STATUS VpScalabilityMultiPipeNext::AddMiFlushDwCmd(
 //!
 //! \param    [in] resource
 //!           Reource used in mi store dat dword cmd
+//! \param    [in] offset
+//!           offset of resource
 //! \param    [in,out] cmdBuffer
 //!           command buffer
 //!
@@ -807,6 +847,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::AddMiFlushDwCmd(
 //!
 MOS_STATUS VpScalabilityMultiPipeNext::AddMiStoreDataImmCmd(
     PMOS_RESOURCE               resource,
+    uint32_t                    offset,
     PMOS_COMMAND_BUFFER         cmdBuffer)
 {
     VP_FUNC_CALL();
@@ -820,7 +861,7 @@ MOS_STATUS VpScalabilityMultiPipeNext::AddMiStoreDataImmCmd(
     auto &params             = m_miItf->MHW_GETPAR_F(MI_STORE_DATA_IMM)();
     params                   = {};
     params.pOsResource       = resource;
-    params.dwResourceOffset  = 0;
+    params.dwResourceOffset  = offset;
     params.dwValue           = 0;
     eStatus                  = m_miItf->MHW_ADDCMD_F(MI_STORE_DATA_IMM)(cmdBuffer);
 
